@@ -202,6 +202,7 @@ ews_notification_dispose (GObject *object)
 		g_signal_handlers_disconnect_by_func (
 			priv->soup_session,
 			ews_notification_authenticate, object);
+		g_clear_object (&priv->soup_session);
 	}
 
 	if (priv->cancellable != NULL)
@@ -318,6 +319,10 @@ e_ews_notification_subscribe_folder_sync (EEwsNotification *notification,
 	g_return_val_if_fail (notification != NULL, FALSE);
 	g_return_val_if_fail (notification->priv != NULL, FALSE);
 
+	/* Can happen during process shutdown */
+	if (!notification->priv->connection)
+		return FALSE;
+
 	msg = e_ews_message_new_with_header (
 		e_ews_connection_get_uri (notification->priv->connection),
 		e_ews_connection_get_impersonate_user (notification->priv->connection),
@@ -329,6 +334,11 @@ e_ews_notification_subscribe_folder_sync (EEwsNotification *notification,
 		FALSE,
 		FALSE);
 
+	if (!msg) {
+		g_warning ("%s: Failed to create Soup message for URI '%s'", G_STRFUNC, e_ews_connection_get_uri (notification->priv->connection));
+		return FALSE;
+	}
+
 	e_soap_message_start_element (msg, "StreamingSubscriptionRequest", "messages", NULL);
 
 	e_soap_message_start_element (msg, "FolderIds", NULL, NULL);
@@ -435,6 +445,10 @@ e_ews_notification_unsubscribe_folder_sync (EEwsNotification *notification,
 	g_return_val_if_fail (notification != NULL, FALSE);
 	g_return_val_if_fail (notification->priv != NULL, FALSE);
 
+	/* Can happen during process shutdown */
+	if (!notification->priv->connection)
+		return FALSE;
+
 	msg = e_ews_message_new_with_header (
 		e_ews_connection_get_uri (notification->priv->connection),
 		e_ews_connection_get_impersonate_user (notification->priv->connection),
@@ -446,6 +460,11 @@ e_ews_notification_unsubscribe_folder_sync (EEwsNotification *notification,
 		FALSE,
 		FALSE);
 
+	if (!msg) {
+		g_warning ("%s: Failed to create Soup message for URI '%s'", G_STRFUNC, e_ews_connection_get_uri (notification->priv->connection));
+		return FALSE;
+	}
+
 	e_ews_message_write_string_parameter_with_attribute (
 		msg, "SubscriptionId", "messages", subscription_id, NULL, NULL);
 
@@ -616,7 +635,8 @@ ews_notification_fire_events_from_response (EEwsNotification *notification,
 	}
 
 	if (events != NULL) {
-		g_signal_emit_by_name (notification->priv->connection, "server-notification", events);
+		if (notification->priv->connection)
+			g_signal_emit_by_name (notification->priv->connection, "server-notification", events);
 		g_slist_free_full (events, (GDestroyNotify) e_ews_notification_event_free);
 	}
 }
@@ -703,9 +723,9 @@ e_ews_notification_get_events_sync (EEwsNotification *notification,
 	gboolean ret;
 	gulong handler_id;
 
-	g_return_val_if_fail (notification != NULL, SOUP_STATUS_CANCELLED);
-	g_return_val_if_fail (notification->priv != NULL, SOUP_STATUS_CANCELLED);
-	g_return_val_if_fail (notification->priv->connection != NULL, SOUP_STATUS_CANCELLED);
+	g_return_val_if_fail (notification != NULL, FALSE);
+	g_return_val_if_fail (notification->priv != NULL, FALSE);
+	g_return_val_if_fail (notification->priv->connection != NULL, FALSE);
 
 	msg = e_ews_message_new_with_header (
 		e_ews_connection_get_uri (notification->priv->connection),
@@ -718,6 +738,11 @@ e_ews_notification_get_events_sync (EEwsNotification *notification,
 		FALSE,
 		FALSE);
 
+	if (!msg) {
+		g_warning ("%s: Failed to create Soup message for URI '%s'", G_STRFUNC, e_ews_connection_get_uri (notification->priv->connection));
+		return FALSE;
+	}
+
 	e_soap_message_start_element (msg, "SubscriptionIds", "messages", NULL);
 	e_ews_message_write_string_parameter_with_attribute (msg, "SubscriptionId", NULL, subscription_id, NULL, NULL);
 	e_soap_message_end_element (msg); /* SubscriptionIds */
